summaryrefslogtreecommitdiffstats
path: root/tty-keys.c
diff options
context:
space:
mode:
authornicm <nicm>2020-05-16 14:16:25 +0000
committernicm <nicm>2020-05-16 14:16:25 +0000
commit9dd58470e41bfb5b9d74028eed73f502e71152f0 (patch)
tree059f11d5a79e7bc9f0d3c954725a96206608e92b /tty-keys.c
parent471f697423df33d535a7cc1fac5f01f718969e34 (diff)
Remove support for iTerm2's DSR 1337 extension and use the CSI > q
extension now supported by a few different terminals.
Diffstat (limited to 'tty-keys.c')
-rw-r--r--tty-keys.c54
1 files changed, 31 insertions, 23 deletions
diff --git a/tty-keys.c b/tty-keys.c
index aa775d69..9e8428f2 100644
--- a/tty-keys.c
+++ b/tty-keys.c
@@ -52,7 +52,7 @@ static int tty_keys_clipboard(struct tty *, const char *, size_t,
size_t *);
static int tty_keys_device_attributes(struct tty *, const char *, size_t,
size_t *);
-static int tty_keys_device_status_report(struct tty *, const char *,
+static int tty_keys_extended_device_attributes(struct tty *, const char *,
size_t, size_t *);
/* Default raw keys. */
@@ -612,8 +612,8 @@ tty_keys_next(struct tty *tty)
goto partial_key;
}
- /* Is this a device status report response? */
- switch (tty_keys_device_status_report(tty, buf, len, &size)) {
+ /* Is this an extended device attributes response? */
+ switch (tty_keys_extended_device_attributes(tty, buf, len, &size)) {
case 0: /* yes */
key = KEYC_UNKNOWN;
goto complete_key;
@@ -936,7 +936,7 @@ tty_keys_clipboard(__unused struct tty *tty, const char *buf, size_t len,
*size = 0;
- /* First three bytes are always \033]52;. */
+ /* First five bytes are always \033]52;. */
if (buf[0] != '\033')
return (-1);
if (len == 1)
@@ -1040,9 +1040,11 @@ tty_keys_device_attributes(struct tty *tty, const char *buf, size_t len,
return (1);
/* Copy the rest up to a 'c'. */
- for (i = 0; i < (sizeof tmp) - 1 && buf[3 + i] != 'c'; i++) {
+ for (i = 0; i < (sizeof tmp) - 1; i++) {
if (3 + i == len)
return (1);
+ if (buf[3 + i] == 'c')
+ break;
tmp[i] = buf[3 + i];
}
if (i == (sizeof tmp) - 1)
@@ -1101,48 +1103,54 @@ tty_keys_device_attributes(struct tty *tty, const char *buf, size_t len,
}
/*
* Handle device status report input. Returns 0 for success, -1 for failure, 1
* for partial.
*/
static int
-tty_keys_device_status_report(struct tty *tty, const char *buf, size_t len,
- size_t *size)
+tty_keys_extended_device_attributes(struct tty *tty, const char *buf,
+ size_t len, size_t *size)
{
struct client *c = tty->client;
u_int i;
char tmp[64];
*size = 0;
- if (tty->flags & TTY_HAVEDSR)
+ if (tty->flags & TTY_HAVEXDA)
return (-1);
- /* First three bytes are always \033[. */
+ /* First four bytes are always \033P>|. */
if (buf[0] != '\033')
return (-1);
if (len == 1)
return (1);
- if (buf[1] != '[')
+ if (buf[1] != 'P')
return (-1);
if (len == 2)
return (1);
- if (buf[2] != 'I' && buf[2] != 'T')
+ if (buf[2] != '>')
return (-1);
if (len == 3)
return (1);
+ if (buf[3] != '|')
+ return (-1);
+ if (len == 4)
+ return (1);
- /* Copy the rest up to a 'n'. */
- for (i = 0; i < (sizeof tmp) - 1 && buf[2 + i] != 'n'; i++) {
- if (2 + i == len)
+ /* Copy the rest up to a '\033\\'. */
+ for (i = 0; i < (sizeof tmp) - 1; i++) {
+ if (4 + i == len)
return (1);
- tmp[i] = buf[2 + i];
+ if (buf[4 + i - 1] == '\033' && buf[4 + i] == '\\')
+ break;
+ tmp[i] = buf[4 + i];
}
if (i == (sizeof tmp) - 1)
return (-1);
- tmp[i] = '\0';
- *size = 3 + i;
+ tmp[i - 1] = '\0';
+ *size = 5 + i;
/* Add terminal features. */
- if (strncmp(tmp, "ITERM2 ", 7) == 0) {
+ if (strncmp(tmp, "ITerm2 ", 7) == 0) {
tty_add_features(&c->term_features,
"256,"
"RGB,"
@@ -1152,7 +1160,7 @@ tty_keys_device_status_report(struct tty *tty, const char *buf, size_t len,
"sync,"
"title",
",");
- } else if (strncmp(tmp, "TMUX ", 5) == 0) {
+ } else if (strncmp(tmp, "tmux ", 5) == 0) {
tty_add_features(&c->term_features,
"256,"
"RGB,"
@@ -1163,10 +1171,10 @@ tty_keys_device_status_report(struct tty *tty, const char *buf, size_t len,
"usstyle",
",");
}
- log_debug("%s: received DSR %.*s", c->name, (int)*size, buf);
+ log_debug("%s: received extended DA %.*s", c->name, (int)*size, buf);
tty_update_features(tty);
- tty->flags |= TTY_HAVEDSR;
+ tty->flags |= TTY_HAVEXDA;
return (0);
}
@@ -1101,48 +1103,54 @@ tty_keys_device_attributes(struct tty *tty, const char *buf, size_t len,
}
/*
- * Handle device status report input. Returns 0 for success, -1 for failure, 1
- * for partial.
+ * Handle extended device attributes input. Returns 0 for success, -1 for
+ * failure, 1 for partial.
*/
static int